#include "HD61202.h"
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "font.h"

uint8_t _x = 0, _y = 0;


/**
  Simple RLE (bzw eine modifizierte RLE) Routine.. Im bergebenen Array stehen pro Arrayelement immer Anzahl Weiss, Anzahl schwarz, Anzahl weiss, etc...
  Dies macht aber nur Sinn, wenn das Bild recht simpel ist, sonst sollte man besser die 8192 bit = 1024 bytes gdirekt in ein Array bernehmen und Anzeigen!
*/
void glcd_rle_picture(const char *progmem_s, uint16_t elements) {
    uint16_t count = 0;
	uint8_t c;
	uint8_t b = 0;
	glcd_clrScr();
	uint16_t pos = 0;
    while (count++ < elements) {
	    c = pgm_read_byte(progmem_s++);
        if (b == 1) {
		  for (uint8_t i = 0; i < c; i++) {
		    glcd_setDot(pos - ((pos / 128) * 128), pos / 128);
		    pos++;
		  }
		}
		else 
		  pos += c;
		b ^= 0x01;    
    }  
 }


void glcd_gotoXY(uint8_t x, uint8_t y) {
    _x = x;
	_y = y;
}

void glcd_putS(char *data) {
  while (*data) {
    glcd_putc(*data);
    data++;
  }
}

void glcd_putc(char chr) {
  uint8_t a = 0, x;

  if (_x > 123) {
		_x = 0;
		_y++;
		if (_y >= 8) _y = 0;
	}
	if (_x < 64){
	    glcd_Control |= (1 << glcd_CS1);
		x = _x;
	} else {
	    glcd_Control |= (1 << glcd_CS2);
		x = _x - 64;
	}
  glcd_setXY(x, _y);
  do {
    glcd_writeData(pgm_read_byte(&font5x7[((chr - 32) * 5) + a]));
    a++;
	_x++;
    if (_x == 64) {
      	    glcd_Control &= ~(1 << glcd_CS1);
      	    glcd_Control |= (1 << glcd_CS2);
    } else if (_x == 128) {
      	    glcd_Control &= ~(1 << glcd_CS2);
      	    glcd_Control |= (1 << glcd_CS1);
      _x = 0;
      _y++;
	if (_y == 8) _y = 0;
    }
    if (_x < 64) x = _x;
		else x = _x - 64;
    glcd_setXY(x, _y);
  } while (a < 5);
  _x++;
  if (_x == 128) _x = 0;
	else glcd_writeData(0x00);
  	    glcd_Control &= ~(1 << glcd_CS1);
  	    glcd_Control &= ~(1 << glcd_CS2);
	glcd_startLine(0);
}

void glcd_setDot(uint8_t x, uint8_t y) {
  uint8_t temp;

	if (x < 64)	{
    glcd_Control |= (1 << glcd_CS1);
    glcd_setXY(x, y / 8);
    temp = glcd_readData();    
    temp = glcd_readData();
    glcd_setXY(x, y / 8);
    glcd_writeData(temp | (1 << (y % 8)));
    glcd_Control &= ~(1 << glcd_CS1);
  } else {
    glcd_Control |= (1 << glcd_CS2);
    glcd_setXY(x - 64, y / 8);
    temp = glcd_readData();    
    temp = glcd_readData();
    glcd_setXY(x - 64, y / 8);
    glcd_writeData(temp | (1 << (y % 8)));
    glcd_Control &= ~(1 << glcd_CS2);
  }
	glcd_startLine(0);
}

void glcd_clrDot(uint8_t x, uint8_t y) {
  uint8_t temp;
	if (x < 64)	{
    glcd_Control |= (1 << glcd_CS1);
    glcd_setXY(x, y / 8);
    temp = glcd_readData();    
    temp = glcd_readData();
    glcd_setXY(x, y / 8);
    glcd_writeData(temp & (0xff - (1 << (y % 8))));
    glcd_Control &= ~(1 << glcd_CS1);
  } else {
    glcd_Control |= (1 << glcd_CS2);
    glcd_setXY(x - 64, y / 8);
    temp = glcd_readData();    
    temp = glcd_readData();
    glcd_setXY(x - 64, y / 8);
    glcd_writeData(temp & (0xff - (1 << (y % 8))));
	    glcd_Control &= ~(1 << glcd_CS2);
  }
	glcd_startLine(0);
}

void glcd_rectangle(uint8_t x, uint8_t y, uint8_t a, uint8_t b) {
  uint8_t j;

  for (j = 0; j < a; j++) {
		glcd_setDot(x, y + j);
		glcd_setDot(x + b - 1, y + j);
	}
  for (j = 0; j < b; j++)	{
		glcd_setDot(x + j, y);
		glcd_setDot(x + j, y + a - 1);
	}
}

void glcd_drawLine(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t color) {
 int x,y,count,xs,ys,xm,ym;
 x=(int)x1; y=(int)y1;
 xs=(int)x2-(int)x1; ys=(int)y2-(int)y1;
 if(xs<0) xm=-1; else if(xs>0) xm=1; else xm=0;
 if(ys<0) ym=-1; else if(ys>0) ym=1; else ym=0;
 if(xs<0) xs=-xs;
 if(ys<0) ys=-ys;
   if (color == 0)
     glcd_clrDot((uint8_t)x,(uint8_t)y);
   else
      glcd_setDot((uint8_t)x,(uint8_t)y);
 if(xs>ys) {
   count=-(xs/2);
   while(x!=x2 ) {
     count=count+ys;
     x=x+xm;
     if(count>0) {
       y=y+ym;
       count=count-xs;
      }
      if (color == 0)
        glcd_clrDot((uint8_t)x,(uint8_t)y);
      else
        glcd_setDot((uint8_t)x,(uint8_t)y);

    }
   }
  else {
    count=-(ys/2);
    while(y!=y2) {
      count=count+xs;
      y=y+ym;
      if(count>0) {
        x=x+xm;
        count=count-ys;
       }
       if (color == 0)
          glcd_clrDot((uint8_t)x,(uint8_t)y);
       else
          glcd_setDot((uint8_t)x,(uint8_t)y);

     }
   }
}

void glcd_init(void)
{
  
  glcd_DataDir = 0x00;
  glcd_Control &= ~(1 << glcd_backLED);  
  glcd_ContDir = ((1 << glcd_RS) | (1 << glcd_RW)  | (1 << glcd_Enable) | (1 << glcd_CS1) |
	     (1 << glcd_CS2)|(1<<glcd_backLED)| (1 << glcd_Reset));
  glcd_Control = ((0 << glcd_RS) | (0 << glcd_RW)  | (0 << glcd_Enable) | (1 << glcd_CS1) |
	     (0 << glcd_CS2)|(1<<glcd_backLED)| (0 << glcd_Reset));
  glcd_Control &= ~(1 << glcd_Reset);  		 
  glcd_delay(1);
  glcd_Control |= (1 << glcd_Reset);    
  glcd_Control |= (1 << glcd_CS1);      
  glcd_waitIfReset();
  glcd_writeCommand(DISPLAY_ON);
  glcd_Control &= ~(1 << glcd_CS1);
  glcd_Control |= (1 << glcd_CS2);
  glcd_waitIfReset();
  glcd_writeCommand(DISPLAY_ON);
  glcd_Control &= ~(1 << glcd_CS2);    
  glcd_clrScr();
}

void glcd_delay(unsigned int p) {                               
  unsigned int i;               
  unsigned char j; 				
  for (i = 0; i < p; i++) for (j = 0; j < 10; j++);
}

void glcd_waitIfBusy(void)
{
  glcd_DataDir = 0x00;
  glcd_Control |= (1 << glcd_RW); 
  glcd_Control &= ~(1 << glcd_RS);      
  do {
    glcd_delay(1);
    glcd_Control |= (1 << glcd_Enable);    	
    glcd_delay(1);
    glcd_Control &= ~(1 << glcd_Enable);    
  } while (bit_is_set(glcd_DataIn, glcd_sBUSY));
}

void glcd_waitIfReset(void)
{
  glcd_DataDir = 0x00;
  glcd_Control |= (1 << glcd_RW);  
  glcd_Control &= ~(1 << glcd_RS);  
  do {
    glcd_delay(1);
    glcd_Control |= (1 << glcd_Enable);    
    glcd_delay(1);
    glcd_Control &= ~(1 << glcd_Enable);    
  } while (bit_is_set(glcd_DataIn, glcd_sRESET));
}

void glcd_writeCommand(unsigned char Command)
{
  glcd_waitIfBusy();
  glcd_Control &= ~(1 << glcd_RW);      
  glcd_Control &= ~(1 << glcd_RS);        
  glcd_DataDir = 0xff;
  glcd_DataOut = Command;
  glcd_Control |= (1 << glcd_Enable);    
  glcd_delay(1);
  glcd_Control &= ~(1 << glcd_Enable);    
}

void glcd_writeData(unsigned char Data)
{
  glcd_waitIfBusy();
  glcd_Control &= ~(1 << glcd_RW); 
  glcd_Control |= (1 << glcd_RS);   
  glcd_DataDir = 0xff;
  glcd_DataOut = Data;
  glcd_Control |= (1 << glcd_Enable);      
  glcd_delay(1);
  glcd_Control &= ~(1 << glcd_Enable);      
}

unsigned char glcd_readData(void) {
  uint8_t result;
  glcd_waitIfBusy();
  glcd_DataDir = 0x00;
  glcd_Control |= (1 << glcd_RW);        
  glcd_Control |= (1 << glcd_RS);          
  glcd_Control |= (1 << glcd_Enable);
  glcd_delay(1);
  glcd_Control &= ~(1 << glcd_Enable);
  result = glcd_DataIn;
  return result;
}


void glcd_setXY(uint8_t x, uint8_t y) {
   glcd_writeCommand(DISPLAY_SET_Y | x);
   glcd_writeCommand(DISPLAY_SET_X | y);   
}

void glcd_startLine(uint8_t start) {
  glcd_Control |= (1 << glcd_CS1);
  glcd_writeCommand(DISPLAY_START_LINE | start);
  glcd_Control &= ~(1 << glcd_CS1);
  glcd_Control |= (1 << glcd_CS2);
  glcd_writeCommand(DISPLAY_START_LINE | start);
  glcd_Control &= ~(1 << glcd_CS2);
}



void glcd_clrScr(void) {
  unsigned char x, y;
  glcd_Control |= (1 << glcd_CS1);
  for (y = 0; y < 8; y++) {
    glcd_setXY(0,y);
    for (x = 0; x < 64; x++) glcd_writeData(0);
  }
  glcd_Control &= ~(1 << glcd_CS1);
  glcd_Control |= (1 << glcd_CS2);
  for (y = 0; y < 8; y++)
	{
    glcd_setXY(0, y);
    for (x = 0; x < 64; x++) glcd_writeData(0);
  }
  glcd_Control &= ~(1 << glcd_CS2);
}
