#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "evalboard.h"
#include "HD61202.h"
#include "logo.h"
#include "uart.h"
#define DEVIDER_PRELOAD 8


unsigned char devider = DEVIDER_PRELOAD;
unsigned char do_animate = 0;
unsigned char address = 0;
unsigned char ticks = 0;
unsigned char leds = 0;
unsigned char x1, x2, y1, y2;
signed char dir1_x = -1;
signed char dir1_y = 1;
signed char dir2_x = -1;
signed char dir2_y = 1;

void animate(void) {
  glcd_drawLine(x1,y1,x2,y2,0);
  if (((x1 + dir1_x) < 0) || ((x1 + dir1_x) > 128))
    dir1_x = -dir1_x;
  if (((x2 + dir2_x) < 0) || ((x2 + dir2_x) > 128))
    dir2_x = -dir2_x;
  if (((y1 + dir1_y) < 0) || ((y1 + dir1_y) > 50))
    dir1_y = -dir1_y;
  if (((y2 + dir2_y) < 0) || ((y2 + dir2_y) > 50))
    dir2_y = -dir2_y;
  x1 += dir1_x; x2 += dir2_x; y1 += dir1_y; y2 += dir2_y;
  glcd_drawLine(x1,y1,x2,y2,1);
}

SIGNAL (SIG_OVERFLOW0) {
//
// Timer0 used for LCD Scrolling Text
//
  char buffer[22];
  devider--;
  if (devider == 0) {
    devider = DEVIDER_PRELOAD;  
    address++;
    ticks++;
    eeprom_read_block(&buffer, (void *) (message + address), 22);
    if (buffer[21] == 0x00)
      address = 0x00;
    if (do_animate == 1) {
      animate();
	}
    buffer[21] = 0x00;
    glcd_gotoXY(0,7);
    glcd_putS(&buffer[0]);
    leds = leds << 1;
    if (leds == 0x08)
      leds = 0x01;
    PORTA = (PORTA & 0xf8) | leds;
	
  }
}    

void init(void) {
  // Init Interrupts
  glcd_init();
  x1 = 5; x2 = 40;
  y2 = 10; y2 = 30;
  DDRA = 0x87;
  leds = 0x01;  
  TCCR0 = 0x05;                            // Timer0 Prescaler
  TIMSK = TIMSK | 0x01;                    // Timer0 Enable
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,XTAL_CPU) );   
  PORTA = PORTA | 0x80 | 0x40 | 0x20;
}

int main (void) {
  unsigned int c;
  cli();  
  init();
  glcd_rle_picture(logo, PIC_ELEMENTS); 
  sei();  
   while (1) {
     while ((ticks % 10) != 9) {
	   if (((PINA & 0x40) == 0) || ((PINA & 0x20) == 0)) {
	      while (((PINA & 0x40) == 0) || ((PINA & 0x20) == 0));
	      PORTA = PORTA ^ 0x80;
	   }
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         uart_puts_P("Received: " );
         uart_putc( (unsigned char) c);
         uart_puts_P("\r\n");
       }
     }
     if (ticks == 69) {
	   do_animate = 1;
	   glcd_clrScr();
	 }
     uart_puts_P("DELTAWAVE AVR Developer !\r\n");
   }
}
