/*************************************************************************************
** SED1335 GLCD Routines   			March, 2006
**
**   						(c) by A.Bhme / Deltawave solutions
**						e-mail: a_boehme@deltawave.de
**						http://www.deltawave.de
**
** Feel free to use (private or commercial) but leave this original comment 
** unchanged !
**  released as-is, no warranty! Tested with AVR MEGA32, 8.8MHz.
***************************************************************************************/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "sed1335.h"



/* 
** constants/macros 
*/
#define PIN(x) (*(&x - 2))
#define DDR(x) (*(&x - 1))

#define glcd_e_delay()   __asm__ __volatile__( "rjmp 1f\n 1:" );
#define glcd_e_high()    GLCD_E_PORT  |=  _BV(GLCD_E_PIN);
#define glcd_e_low()     GLCD_E_PORT  &= ~_BV(GLCD_E_PIN);
#define glcd_res_high()  GLCD_RES_PORT |=  _BV(GLCD_RES_PIN)
#define glcd_res_low()   GLCD_RES_PORT &= ~_BV(GLCD_RES_PIN)
#define glcd_cs_high()   GLCD_CS_PORT |=  _BV(GLCD_CS_PIN)
#define glcd_cs_low()    GLCD_CS_PORT &= ~_BV(GLCD_CS_PIN)
#define glcd_rw_high()   GLCD_RW_PORT |=  _BV(GLCD_RW_PIN)
#define glcd_rw_low()    GLCD_RW_PORT &= ~_BV(GLCD_RW_PIN)
#define glcd_a0_high()   GLCD_A0_PORT |=  _BV(GLCD_A0_PIN)
#define glcd_a0_low()    GLCD_A0_PORT &= ~_BV(GLCD_A0_PIN)
#define glcd_bl_high()   GLCD_BL_PORT |=  _BV(GLCD_BL_PIN)
#define glcd_bl_low()    GLCD_BL_PORT &= ~_BV(GLCD_BL_PIN)


/*************************************************************************
 delay loop for small accurate delays: 16-bit counter, 4 cycles/loop
*************************************************************************/
static inline void _delayFourCycles(unsigned int __count)
{
    if ( __count == 0 )    
        __asm__ __volatile__( "rjmp 1f\n 1:" );    
    else
        __asm__ __volatile__ (
    	    "1: sbiw %0,1" "\n\t"                  
    	    "brne 1b"                   
    	    : "=w" (__count)
    	    : "0" (__count)
    	   );
}


/************************************************************************* 
delay for a minimum of <us> microseconds
the number of loops is calculated at compile-time from MCU clock frequency
*************************************************************************/
#define delay(us)  _delayFourCycles( ( ( 1*(XTAL/4000) )*us)/1000 )

void glcd_backlight(uint8_t command) {
  if (command == ON)
     glcd_bl_high();
  else
    glcd_bl_low();
}

void glcd_data_write(uint8_t data) {
  GLCD_DATA_PORT = data;
  glcd_a0_low();
  glcd_rw_low();
  glcd_e_high();
  glcd_e_delay();
  glcd_e_low();
}

uint8_t glcd_data_read(void) {
  uint8_t data;
  DDR(GLCD_DATA_PORT) = 0x00;
  glcd_a0_high();
  glcd_rw_high();
  glcd_e_high();
  glcd_e_delay()
  data = PIN(GLCD_DATA_PORT);
  glcd_e_low();
  DDR(GLCD_DATA_PORT) = 0xff  ;
  return data;
}


void glcd_command_write(uint8_t command) {
  GLCD_DATA_PORT = command;
  glcd_a0_high();
  glcd_rw_low();
  glcd_e_high();
  glcd_e_delay();
  glcd_e_low();
}

void glcd_setpos(uint16_t pos) {
  glcd_command_write(GLCD_CMD_SETCURSOR);     //Set Cursor
  pos += GLCD_GFX_ADR;
  glcd_data_write(pos & 0xff);
  glcd_data_write(pos / 256);
}

void glcd_gotoxy(uint8_t x, uint8_t y) {
    uint16_t offset;
	offset = GLCD_TEXT_ADR + y * GLCD_TEXT_RES_X + x;
    glcd_command_write(GLCD_CMD_SETCURSOR);     //Set Cursor
    glcd_data_write(offset & 0xff);
    glcd_data_write(offset / 256);
}

void glcd_clrscr(uint8_t area) {
  if ((area & 0x02) == 0x02) {
     glcd_command_write(GLCD_CMD_SETCURSOR);     //Set Cursor
      glcd_data_write(GLCD_GFX_ADR & 0xff);
      glcd_data_write(GLCD_GFX_ADR / 256);
      glcd_command_write(GLCD_CMD_MWRITE);  
      for (unsigned int i = 0; i < ((GLCD_RES_X * GLCD_RES_Y) / 8); i++)
	    glcd_data_write(0x00);
  }
  if ((area & 0x01) == 0x01) {
      glcd_gotoxy(0,0);
      glcd_command_write(GLCD_CMD_MWRITE);  
      for (unsigned int i = 0; i < 256; i++)
         glcd_data_write(0x00);
      glcd_gotoxy(0,0);   
  }
}

/*
 * Zeigt einen String an pos x,y an
 */
void glcd_puts_xy(const char *txt, uint8_t xpos, uint8_t ypos) {
  char ps;
  glcd_gotoxy(xpos,ypos);
  glcd_command_write(0x42);  
 while((ps = *txt)) {
   txt++; //Nchstes Zeichen
   if (ps== 0) break; //Raus wenn Zeichen gleich 0
   glcd_data_write(ps);
  }
}

/*
 * String aus PROGMEM(Flash) anzeigen
 */
void glcd_puts_xy_p(const char *progmem_s, uint8_t xpos, uint8_t ypos) {
    unsigned char c;
	glcd_gotoxy(xpos,ypos);
    glcd_command_write(0x42);  	
    while ( (c = pgm_read_byte(progmem_s++)) ) {
       glcd_data_write(c);
    }  
}

void glcd_set_pixel(uint8_t x, uint8_t y, uint8_t color) {
    uint16_t pos;
    uint8_t bits;
    pos = (y * ( GLCD_RES_Y / 8)) + (x / 8);
    glcd_setpos(pos);
	glcd_command_write(GLCD_CMD_MREAD);
    bits = glcd_data_read();
	if (color == GLCD_BLACK)
      bits |= (0x80 >> (x & 0x07)); 
	else
      bits &= ~(0x80 >> (x & 0x07)); 
    glcd_setpos(pos);
	glcd_command_write(GLCD_CMD_MWRITE);	
    glcd_data_write(bits); 	
}

void glcd_draw_line(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t color) {
 int x,y,count,xs,ys,xm,ym;
 x=(int)x1; y=(int)y1;
 xs=(int)x2-(int)x1; ys=(int)y2-(int)y1;
 if(xs<0) xm=-1; else if(xs>0) xm=1; else xm=0;
 if(ys<0) ym=-1; else if(ys>0) ym=1; else ym=0;
 if(xs<0) xs=-xs;
 if(ys<0) ys=-ys;
 glcd_set_pixel((uint8_t)x,(uint8_t)y,color);
 if(xs>ys) {
   count=-(xs/2);
   while(x!=x2 ) {
     count=count+ys;
     x=x+xm;
     if(count>0) {
       y=y+ym;
       count=count-xs;
      }
     glcd_set_pixel((uint8_t)x,(uint8_t)y,color);
    }
   }
  else {
    count=-(ys/2);
    while(y!=y2) {
      count=count+xs;
      y=y+ym;
      if(count>0) {
        x=x+xm;
        count=count-ys;
       }
      glcd_set_pixel((uint8_t)x,(uint8_t)y,color);
     }
   }
}


void glcd_rle_picture(const char *progmem_s, uint16_t elements) {
    uint16_t count = 0;
	uint8_t c;
	uint8_t b = 0;
	uint16_t pos = 0;
    while (count++ < elements) {
	    c = pgm_read_byte(progmem_s++);
        if (b == 1) {
		  for (uint8_t i = 0; i < c; i++) {
		    glcd_set_pixel(pos - ((pos / GLCD_RES_X) * GLCD_RES_X), pos / GLCD_RES_X, GLCD_BLACK);
		    pos++;
		  }
		}
		else 
		  pos += c;
		b ^= 0x01;    
    }  
 }

void glcd_init(void) {
  DDR(GLCD_DATA_PORT) = 0xff;
  DDR(GLCD_RES_PORT)    |= _BV(GLCD_RES_PIN);
  DDR(GLCD_CS_PORT)    |= _BV(GLCD_CS_PIN);  
  DDR(GLCD_E_PORT)    |= _BV(GLCD_E_PIN);    
  DDR(GLCD_RW_PORT)    |= _BV(GLCD_RW_PIN);
  DDR(GLCD_A0_PORT)    |= _BV(GLCD_A0_PIN);    
  DDR(GLCD_BL_PORT)    |= _BV(GLCD_BL_PIN);    
  glcd_bl_low();
  glcd_res_low();
  delay(60000); 
  glcd_res_high();
  delay(60000); 

  glcd_command_write(GLCD_CMD_SYSTEMSET);
#if (GLCD_PANELS + 0) > 1
  glcd_data_write(0x38);	 // P1
#else
  glcd_data_write(0x30);	 // P1
#endif
  glcd_data_write(0x80 + (GLCD_CHAR_WIDTH - 1));  //P2
  glcd_data_write(GLCD_CHAR_HEIGHT - 1);    //P3
  glcd_data_write(GLCD_TEXT_RES_X - 1);  //P4
  glcd_data_write(GLCD_TEXT_RES_X + 3);  //P5
  glcd_data_write(GLCD_RES_Y - 1 );
  glcd_data_write(GLCD_TEXT_RES_X);
  glcd_data_write(0x00);

   glcd_command_write(GLCD_CMD_SCROLL);     //Scroll
   glcd_data_write(GLCD_TEXT_ADR & 0xff);
   glcd_data_write(GLCD_TEXT_ADR / 256);
   glcd_data_write(GLCD_RES_Y / GLCD_PANELS);  
   glcd_data_write(GLCD_GFX_ADR & 0xff);
   glcd_data_write(GLCD_GFX_ADR / 256);
   glcd_data_write(GLCD_RES_Y / GLCD_PANELS);     //SL2
   glcd_data_write((GLCD_TEXT_ADR + (GLCD_TEXT_RES_X * GLCD_TEXT_RES_Y / 2)) & 0xff);   
   glcd_data_write((GLCD_TEXT_ADR + (GLCD_TEXT_RES_X * GLCD_TEXT_RES_Y) / 2) / 256);  
   glcd_data_write((GLCD_GFX_ADR + (GLCD_RES_X * GLCD_RES_Y / 2 / 8)) & 0xff); 
   glcd_data_write((GLCD_GFX_ADR + (GLCD_RES_X * GLCD_RES_Y / 2/ 8)) / 256); 

   glcd_command_write(0x4C);     
   glcd_command_write(0x5A);     
   glcd_data_write(0x00);

   glcd_command_write(0x5B);     
   glcd_data_write(0x01);  

   glcd_command_write(0x5D);     
   glcd_data_write(0x05);
   glcd_data_write(0x86);

   glcd_command_write(0x5C);     
   glcd_data_write(0x00);
   glcd_data_write(30);

   glcd_command_write(0x59);
   glcd_data_write(0x57);

   glcd_clrscr(GLCD_COMPLETEAREA);
   
}


