#ifndef SED1335_H
#define SED1335_H 1
#include <inttypes.h>
#include <avr/pgmspace.h>

#define XTAL 8867238              /**< clock frequency in Hz, used to calculate delay timer */


/** Connection parameters
**/
#define GLCD_DATA_PORT   PORTC
#define GLCD_RES_PORT    PORTD
#define GLCD_RES_PIN     2    
#define GLCD_CS_PORT     PORTD
#define GLCD_CS_PIN      3    
#define GLCD_E_PORT     	PORTD
#define GLCD_E_PIN      	4       
#define GLCD_RW_PORT		PORTD
#define GLCD_RW_PIN		5
#define GLCD_A0_PORT		PORTD
#define GLCD_A0_PIN		6
#define GLCD_BL_PORT		PORTD		// PORT and PIN for Backlight control (optional)
#define GLCD_BL_PIN		7

/** GLCD parameters
**
** Tested with 128x128 DUALPANEL Display, calc of other parameters has to be checked !!
 **/
#define GLCD_RES_X 128
#define GLCD_RES_Y 128
#define GLCD_CHAR_WIDTH 8
#define GLCD_CHAR_HEIGHT 8
#define GLCD_TEXT_RES_X (GLCD_RES_X / GLCD_CHAR_WIDTH)
#define GLCD_TEXT_RES_Y (GLCD_RES_Y / GLCD_CHAR_HEIGHT)
#define GLCD_PANELS 2
#define GLCD_TEXT_ADR 0x0
#define GLCD_GFX_ADR 0x800

#define ON 1
#define OFF 0
#define GLCD_CMD_SYSTEMSET	0x40
#define GLCD_CMD_MWRITE 0x42
#define GLCD_CMD_MREAD 0x43
#define GLCD_CMD_SCROLL 0x44
#define GLCD_CMD_SETCURSOR 0x46

/** Text / Graphic Layer select **/
#define GLCD_TEXTAREA 1
#define GLCD_GFXAREA 2
#define GLCD_COMPLETEAREA 3

#define GLCD_BLACK 0
#define GLCD_WHITE 1

extern void glcd_data_write(uint8_t data);
extern uint8_t glcd_data_read(void);
extern void glcd_command_write(uint8_t command);  
extern void glcd_init(void);
extern void glcd_clrscr(uint8_t area);
extern void glcd_gotoxy(uint8_t x, uint8_t y);
extern void glcd_puts_xy(const char *txt, uint8_t xpos, uint8_t ypos);
extern void glcd_puts_xy_p(const char *progmem_s, uint8_t xpos, uint8_t ypos);
#define glcd_puts_xy_P(__s,__x,__y) glcd_puts_xy_p(PSTR(__s),__x,__y)
void glcd_set_pixel(uint8_t x, uint8_t y, uint8_t color);
extern void glcd_draw_line(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t color);
extern void glcd_rle_picture(const char *progmem_s, uint16_t elements) ;
#endif
