#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "lcd.h"

#define lcd_wr_high()    LCD_WR_PORT  |=  _BV(LCD_WR_PIN);
#define lcd_wr_low()     LCD_WR_PORT  &= ~_BV(LCD_WR_PIN);
#define lcd_rd_high()    LCD_RD_PORT  |=  _BV(LCD_RD_PIN);
#define lcd_rd_low()     LCD_RD_PORT  &= ~_BV(LCD_RD_PIN);
#define lcd_ce_high()    LCD_CE_PORT  |=  _BV(LCD_CE_PIN);
#define lcd_ce_low()     LCD_CE_PORT  &= ~_BV(LCD_CE_PIN);
#define lcd_cd_high()    LCD_CD_PORT  |=  _BV(LCD_CD_PIN);
#define lcd_cd_low()     LCD_CD_PORT  &= ~_BV(LCD_CD_PIN);
#define lcd_rst_high()    LCD_RST_PORT  |=  _BV(LCD_RST_PIN);
#define lcd_rst_low()     LCD_RST_PORT  &= ~_BV(LCD_RST_PIN);

#define PIN(x) (*(&x - 2))  /* address of data direction register of port x */
#define DDR(x) (*(&x - 1))  /* address of input register of port x          */



/*
 *  Display lsst sich auch im "Spiegelmodus" betreiben, d.h. die Datenleitungen sind ungekehrt
 * um die Verbindungen auf der Platine einfacher zu gestalten. Diese Funktion kehrt die Bits in
 * einem Byte um
*/
#if LCD_DATA_REVERSE
#define reverse(a) ({ \
           unsigned char ch; \
           asm volatile ( \
           "ror %1" "\n\t" \
           "rol %0" "\n\t" \
           "ror %1" "\n\t" \
           "rol %0" "\n\t" \
           "ror %1" "\n\t" \
           "rol %0" "\n\t" \
           "ror %1" "\n\t" \
           "rol %0" "\n\t" \
           "ror %1" "\n\t" \
           "rol %0" "\n\t" \
           "ror %1" "\n\t" \
           "rol %0" "\n\t" \
           "ror %1" "\n\t" \
           "rol %0" "\n\t" \
           "ror %1" "\n\t" \
           "rol %0" "\n\t" \
           "ror %1"  /* restore input register */ \
           : /* output */ "=r&" (ch) \
           : /* input */ "r" ((uint8_t)(a)) \
           ); \
           ch; \
   })
#endif

void lcd_drawLine(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char color) {
 int x,y,count,xs,ys,xm,ym;
 x=(int)x1; y=(int)y1;
 xs=(int)x2-(int)x1; ys=(int)y2-(int)y1;
 if(xs<0) xm=-1; else if(xs>0) xm=1; else xm=0;
 if(ys<0) ym=-1; else if(ys>0) ym=1; else ym=0;
 if(xs<0) xs=-xs;
 if(ys<0) ys=-ys;
 lcd_setPixel((unsigned char)x,(unsigned char)y,color);
 if(xs>ys) {
   count=-(xs/2);
   while(x!=x2 ) {
     count=count+ys;
     x=x+xm;
     if(count>0) {
       y=y+ym;
       count=count-xs;
      }
     lcd_setPixel((unsigned char)x,(unsigned char)y,color);
    }
   }
  else {
    count=-(ys/2);
    while(y!=y2) {
      count=count+xs;
      y=y+ym;
      if(count>0) {
        x=x+xm;
        count=count-ys;
       }
      lcd_setPixel((unsigned char)x,(unsigned char)y,color);
     }
   }
}


/*
 * Zeichensatz fr Display convertieren
 */
unsigned char convert(unsigned char ch) {
 unsigned char tmp;
 tmp=ch;
 switch(ch)
  {
   case 196: tmp=0x6E; break; // 
   case 228: tmp=0x64; break; // 
   case 214: tmp=0x79; break; // 
   case 246: tmp=0x74; break; // 
   case 220: tmp=0x7A; break; // 
   case 252: tmp=0x61; break; // 
   case 223: tmp=223; break; // 
   default: tmp-=0x20; break;
  }
 return tmp;
}



/*
 * Zeigt einen String an pos x,y an
 */
void lcd_puts_xy(const char *txt, unsigned char xpos, unsigned char ypos) {
 char ps;
 lcd_data2(xpos+(cGraphColumns)*ypos,0x24);
 while((ps = *txt)) {
   txt++; //Nchstes Zeichen
   if (ps== 0) break; //Raus wenn Zeichen gleich 0
   ps=convert(ps); //bersetzungstabelle aufrufen
   lcd_data1(ps,0xC0); //Write ps, Auto Inkrement
  }
}

/*
 * String aus PROGMEM(Flash) anzeigen
 */
void lcd_puts_xy_p(const char *progmem_s, unsigned char xpos, unsigned char ypos) {
    unsigned char c;
	lcd_data2(xpos+(cGraphColumns)*ypos,0x24);
    while ( (c = pgm_read_byte(progmem_s++)) ) {
        lcd_data1(convert(c),0xC0);
    }  
}


/*
 * Zeigt eine Zahl in Dezimaldarstellung auf LCD an
 */
void printFormat(unsigned int i) {
    unsigned char bcd;
    unsigned char j;
    static unsigned int n[] = {10000, 1000, 100, 10, 1};
    for (j = 0; j < 5; j++)
      {
        bcd = '0';
        while (i > n[j])
          {
            i -= n[j];
            bcd++;
          }
        lcd_data1(bcd-0x20,0xC0);
      }
}


/*
 * LCD initialisieren und lschen
 */
void lcd_init(void) {
  DDR(LCD_WR_PORT)    |= _BV(LCD_WR_PIN);
  DDR(LCD_RD_PORT)    |= _BV(LCD_RD_PIN);
  DDR(LCD_CE_PORT)    |= _BV(LCD_CE_PIN);
  DDR(LCD_CD_PORT)    |= _BV(LCD_CD_PIN);
  DDR(LCD_RST_PORT)    |= _BV(LCD_RST_PIN);
  lcd_rst_low();
  lcd_rst_high();
  lcd_rd_high();
  lcd_wr_high();
  lcd_ce_high();
  // Display init
  lcd_data2(0, 0x40);
  lcd_data2(cGraphColumns, 0x41);
  lcd_data2(0x0200, 0x42);
  lcd_data2(cGraphColumns, 0x43);
  lcd_command(0x80);
  lcd_command(0x9f);
  lcd_data2(0x0002, 0x22);
  lcd_clrscr();
 }


/*
 * Ein Pixel auf LCD setzen bzw. entfernen
 */
void lcd_setPixel(unsigned char xpos, unsigned char ypos, unsigned char mode)
{
 unsigned int adress;
 unsigned char tmp,pixel;
 adress = 0x0200;
 adress+= cGraphColumns * ypos;
 adress+= xpos / cFontWidth;
 lcd_data2(adress, 0x24);
 lcd_data((unsigned char)(adress&0xFF)); //Daten schreiben
 lcd_data((unsigned char)(adress>>8)); //Daten schreiben
 lcd_command(0x24); //Kommando schreiben
 if(mode==0) pixel=0xF0; //Kommando Pixel lschen
 else pixel=0xF8;        //Kommando Pixel setzen

//Die Reihenfolge der Pixel ist im Byte genau umgekehrt !
 tmp=(cFontWidth-1)-(xpos%cFontWidth);
 pixel|=tmp; //Position des Pixels im Byte setzen

 lcd_command(pixel); //Befehl mit gewnschtem Pixel schreiben
}



/*
 * Speicher Image einer Grafik (-> converter.java) wird in das Display geschoben
 */
void lcd_dsp_picture_p(const char *progmem_s, uint16_t elements) {
    unsigned char c;
	unsigned int count = 0;
	lcd_data2(0x200,0x24);
    while (count++ < elements) {
	    c = pgm_read_byte(progmem_s++);
        lcd_data1(c,0xC0);
    }  
 }



void lcd_clrGraphics(void) {
  lcd_data(0);
  lcd_data(2);
  lcd_command(0x24);
  for (int z = 0; z < (cGraphResY*((cGraphResX+(cFontWidth-1)) / cFontWidth)); z++) {
    lcd_data(0);
    lcd_command(0xC0);
  }
}

/*
 * LCD lschen
 */
void lcd_clrscr(void) {
  // Text lschen
  lcd_data(0);
  lcd_data(0);
  lcd_command(0x24);
  for (int z = 0; z <= (cTxtResY*cGraphColumns); z++) {
    lcd_data(0);
    lcd_command(0xC0);
  }
 lcd_clrGraphics();
}


/*
 * LCD Handshake
 */
static void waitLCD(void) {
  unsigned char data; 
  DDR(LCD_DATA) = 0x00;
  lcd_cd_high();
  lcd_wr_high();
  lcd_rd_high();
  do {
    lcd_ce_low();
	lcd_rd_low();    	
	asm volatile ("nop");
	#if LCD_DATA_REVERSE
	  data = reverse(PIN(LCD_DATA));
	#else
	   data = PIN(LCD_DATA);
    #endif
    lcd_rd_high();  
    lcd_ce_high();  
  } while (((data & 0x03) != 0x03));
}


/*
 * Byte an LCD bertragen
 */
static void lcd_write(uint8_t data, uint8_t cd) {
  DDR(LCD_DATA) = 0xff;
  #if LCD_DATA_REVERSE
    LCD_DATA = reverse(data);
  #else
    LCD_DATA = data;
  #endif
  if (cd) {
    lcd_cd_high();
  }
  else {
    lcd_cd_low();
  }
  lcd_ce_low();
  lcd_rd_high();
  lcd_wr_low();
  lcd_wr_high();
  lcd_ce_high();
}

void lcd_data(uint8_t data) {
  waitLCD();
  lcd_write(data, 0);
}

void lcd_data2(uint16_t data, uint8_t cmd) {
  lcd_data((unsigned char)(data & 0xff));
  lcd_data((unsigned char)(data >> 8));
  lcd_command(cmd);
}

void lcd_data1(uint8_t data, uint8_t cmd) {
  lcd_data(data);
  lcd_command(cmd);
}


void lcd_command(uint8_t command) {
  waitLCD();
  lcd_write(command, 1);
}
