#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "evalboard.h"
#include "lcd.h"
#include "uart.h"
#include "i2cmaster.h"
#define DEVIDER_PRELOAD 8

unsigned char address = 0;
unsigned char ticks = 0;
unsigned char devider = DEVIDER_PRELOAD;
unsigned char leds = 0x04;

unsigned char checkTaster(void) {
     unsigned char result = 0;
     DDRD = DDRD & 0x9F;
     PORTD = PORTD | 0x60;
     if ((PIND & 0x40) == 0x00) {
         uart_puts_P("Taster 1\r\n");
         lcd_puts_P("Taster 1        ");
         result = 1;
     }
     if ((PIND & 0x20) == 0x00) {
         lcd_puts_P("Taster 2        " );
         uart_puts_P("Taster 2\r\n");
         result = 1;         
     }    
     return result;
}


SIGNAL (SIG_OVERFLOW0) {
//
// Timer0 used for LCD Scrolling Text
//
  char buffer[17];
  devider--;
  if (devider == 0) {
    devider = DEVIDER_PRELOAD;
    address++;
    ticks++;
    eeprom_read_block(&buffer, (void *) (message + address), 17);
    if (buffer[16] == 0x00)
      address = 0x00;
    buffer[16] = 0x00;
    lcd_gotoxy(0,0);
    if (!checkTaster())
      lcd_puts(&deltawave[address % 2][0]);
    lcd_gotoxy(0,1);
    lcd_puts(&buffer[0]);
    leds = leds << 1;
    if (leds >= 0x20)
      leds = 0x04;
    DDRD = DDRD | 0x1c;
    PORTD = (PORTD & 0xe3) | leds;
  }
}



void init(void) {
  // Init Interrupts
  lcd_init(LCD_DISP_ON);    
  lcd_clrscr();   
  TCCR0 = 0x05;                            // Timer0 Prescaler
  TIMSK = TIMSK | 0x01;                    // Timer0 Enable
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,XTAL_CPU) );   
  sei();   
}

/*
 * ein Byte des 24Cxx an Adresse address auslesen
 *
*/
unsigned char read_24c(unsigned int address) {
        unsigned char ret;
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write((address >> 8) & 0xff);
        i2c_write(address & 0xff);
        i2c_rep_start(i24c64+I2C_READ);       // set device address and read mode
        ret = i2c_readNak();                    // read one byte         
        i2c_stop();     
        return ret;
}


void write_24c(unsigned int address, unsigned char data) {
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write((address >> 8) & 0xff);
        i2c_write(address & 0xff);
        i2c_write(data);
        i2c_stop();
     }



void downloadEEprom(void) {
        unsigned char ret = 0;
	unsigned long address_count = 0;
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write(0x00);
        i2c_write(0x00);
        i2c_rep_start(i24c64+I2C_READ);       // set device address and read mode
        do {
          ret = i2c_readAck();                    // read one byte 
          uart_putc(ret);
	  address_count++;
        } while ((ret != 0) && (ret != 0xff) && (address_count < i24cxx_max));
        ret = i2c_readNak();                    // read one byte         
        i2c_stop();     
} 


void serialProgram(void) {
     unsigned int c;
     unsigned int adr = 0;
     while (ticks < 10) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         write_24c(adr++, c & 0xff);
         ticks = 0;
       }
     }
}

 /*
 * Kleines Demoprogramm fr das MEGA32 System v2
 * Steuerung ber serielle Schnittstelle wie folgt:
             d  - Inhalt des EEProms (24cxx) seriell downloaden
             w  - Text, welcher ber serielle Schnittstelle empfangen wird, in EEPROM
                  speichern (10 ticks timeout !!) 
*/
int main(void) {
   unsigned int c;
   init();
   while (1) {
     while (ticks < 10) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
          switch (c) {
            case 'd': downloadEEprom(); break;
            case 'w': serialProgram(); break;
			default: uart_puts_P("Received: ");
			         uart_putc(c);
					 uart_puts_P("\r\n");
          }
       }
     }
     ticks = 0;
     uart_puts_P("DELTAWAVE AVR Developer !\r\n");
   }
}

