#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "evalboard.h"
#include "lcd.h"
#include "uart.h"

unsigned char address = 0;
unsigned char ticks = 0;

SIGNAL (SIG_OVERFLOW0) {
//
// Timer0 used for LCD Scrolling Text
//
  char buffer[17];
  ticks++;
  if ((ticks % 16) == 0) {
    address++;
    eeprom_read_block(&buffer, (void *) (message + address), 17);
    if (buffer[16] == 0x00)
      address = 0x00;
    buffer[16] = 0x00;
    lcd_gotoxy(0,0);
    lcd_puts(&deltawave[address % 2][0]);
    lcd_gotoxy(0,1);
    lcd_puts(&buffer[0]);
  }
}    

void init(void) {
  // Init Interrupts
  lcd_init(LCD_DISP_ON);    
  lcd_clrscr();   
  DDRA = 0xff;
  TCCR0 = 0x05;                            // Timer0 Prescaler
  TIMSK = TIMSK | 0x01;                    // Timer0 Enable
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,XTAL_CPU) );   
  sei();   
}


int main(void) {
   unsigned int c;
   init();
   while (1) {
     while (ticks < 160) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         uart_puts_P("Received: " );
         uart_putc( (unsigned char) c);
         uart_puts_P("\r\n");
       }
     }
     ticks = 0;
     uart_puts_P("AVR Entwickler Board\r\n");
   }
}
