#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "evalboard.h"
#include "lcd.h"
#include "uart.h"
#include "i2cmaster.h"
#include "rtc4513.h"

unsigned char address = 0;
unsigned char ticks = 0;
unsigned char leds = 0;

SIGNAL (SIG_OVERFLOW0) {
//
// Timer0 used for LCD Scrolling Text
//
  char buffer[17];
  address++;
  ticks++;
  eeprom_read_block(&buffer, (void *) (message + address), 17);
  if (buffer[16] == 0x00)
    address = 0x00;
  buffer[16] = 0x00;
  lcd_gotoxy(0,0);
  lcd_puts(&deltawave[address % 2][0]);
  lcd_gotoxy(0,1);
  lcd_puts(&buffer[0]);
  leds = leds << 1;
  if (leds == 0x08)
    leds = 0x01;
  PORTA = (PORTA & 0xf8) | leds;
}    

/*
 * uebergebene Zahl i als dezimalzahl anzeigen (target = 0 -> lcd, =1 -> seriell)
 * an der stelle dp wird ein Punkt gesetzt (Float emulation) dp > decimals -> kein Punkt
 * decimals bestimmt, wieviele Stellen angezeigt werden
 * printFormat(12345, 2, 5, 1) gibt 12.345 auf LCD aus
*/
void printFormat(int i, unsigned char dp, unsigned char start, unsigned char decimals, unsigned char target) {
    unsigned char bcd;
    unsigned char j;
    static unsigned int n[] = {10000, 1000, 100, 10, 1};
    if (i < 0) {
      if (target)
        uart_puts_P("-");
      else
        lcd_puts_P("-");
      i = -i;
    }
    for (j = start; j < decimals; j++) {
        bcd = '0';
        while (i >= n[j]) {
            i -= n[j];
            bcd++;
         }
		if (j == dp) {
          if (target)
  		    uart_puts_P(".");
          else
            lcd_puts_P(".");
         }
         if (target)
          uart_putc(bcd);
         else
          lcd_putc(bcd);
      }
}

void init(void) {
  // Init Interrupts
  lcd_init(LCD_DISP_ON);    
  lcd_clrscr();   
  leds = 0x01;
  DDRA = 0xff;
  TCCR0 = 0x05;                            // Timer0 Prescaler
  TIMSK = TIMSK | 0x01;                    // Timer0 Enable
  rtc_init();
  DDRD = DDRD & 0xE7;                       // Taster als Input
  DDRD = DDRD | 0x04;                       // Relais muss Ausgang sein
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,XTAL_CPU) );   
  sei();   
}


/*
 * Setzt den Port des PCF8574 IO-Expanders
 * data wird an den Ausgang gelegt
 *
*/
void out_pcf(unsigned char data) {
        i2c_init();
        i2c_start_wait(pcf8574+I2C_WRITE);     // PCF Addressieren
        i2c_write(data);                       // und schreiben
        i2c_stop();
     }


/*
 * ein Byte des 24Cxx an Adresse address auslesen
 *
*/
unsigned char read_24c(unsigned int address) {
        unsigned char ret;
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write((address >> 8) & 0xff);
        i2c_write(address & 0xff);
        i2c_rep_start(i24c64+I2C_READ);       // set device address and read mode
        ret = i2c_readNak();                    // read one byte         
        i2c_stop();     
        return ret;
}


/*
 * ein Byte des 24Cxx an Adresse address schreiben
 *
*/
void write_24c(unsigned int address, unsigned char data) {
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write((address >> 8) & 0xff);
        i2c_write(address & 0xff);
        i2c_write(data);
        i2c_stop();
     }

/*
 * Inhalt des 24Cxx als Text seriell downloaden
 * Der Download bricht ab, sobald ein 0xff oder 0x00 im EEprom steht
 *
*/

void downloadEEprom(void) {
        unsigned char ret = 0;
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write(0x00);
        i2c_write(0x00);
        i2c_rep_start(i24c64+I2C_READ);       // set device address and read mode
        do {
          ret = i2c_readAck();                    // read one byte 
          uart_putc(ret);
        } while ((ret != 0) && (ret != 0xff));
        ret = i2c_readNak();                    // read one byte         
        i2c_stop();     
}

/*
 * Zeichen entweder an UART oder LCD ausgeben
*/
void put_c(unsigned char data, unsigned char target) {
     if (target)
       uart_putc(data);
     else
       lcd_putc(data);
     }

/*
 * Zeit aus RTC auslesen und formatiert anzeigen
*/
void printTime(unsigned char target) {
    unsigned char time[3]; 
    rtc_set_bus();
    gettime(time);
    printFormat(time[0], 6,3,5, target); put_c(':', target); printFormat(time[1], 6,3,5, target); put_c(':', target); printFormat(time[2], 6,3,5, target);
    put_c(0x0a, target); put_c(0x0d, target);  
}

/*
 * Taster an Port D berprfen und ber serielle Schnittstelle Status ausgeben
*/
void checkTaster(void) {
       DDRD = DDRD & 0xE7;                       // Taster als Input
       PORTD = PORTD | 0x18;                     // Pull-Ups
       if ((PIND & 0x08) == 0x00)
         uart_puts_P("Taster 1\r\n");
       if ((PIND & 0x10) == 0x00)
         uart_puts_P("Taster 2\r\n");
     }

/*
 * Relais seriell steuern kommando r1 -> ein, r0 -> aus
*/
void toggle_relais(void) {
     unsigned int c;
     ticks = 0;
     while (ticks < 5) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         if (c == '0') PORTD = PORTD & 0xfb;
         if (c == '1') PORTD = PORTD | 0x04;
         ticks = 5;
       }
     }
}

/*
 * PCF IO-Expander seriell steuern
*/
void toggle_pcf(void) {
     unsigned int c;
     ticks = 0;
     while (ticks < 5) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         if (c == '0') out_pcf(0x00);
         if (c == '1') out_pcf(0xff);
         ticks = 5;
       }
     }
}

/*
 * RTC ber serielle Schnittstelle setzten
 * Kommando: shhmmss (s=Kommando hh=Stunde mm=Minute ss=Sekunde)
*/
void setClock(void) {
     unsigned int c;
     unsigned char dataadr = 0;
     unsigned char data[6];
     ticks = 0;
     while ((ticks < 5) && (dataadr < 6)) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         data[dataadr] = (unsigned char) c - 0x30;
         dataadr++;
         ticks = 0;
       }
     }
     settime(data[0] * 10 + data[1], data[2] * 10 + data[3], data[4] * 10 + data[5]);
}

/*
 * Serielle Daten in EEProm abspeicher, ab Adresse 0x0000
 * TIMEOUT beachten!!
*/
void serialProgram(void) {
     unsigned int c;
     unsigned int adr = 0;
     while (ticks < 10) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         write_24c(adr++, c & 0xff);
         ticks = 0;
       }
     }
     
     }

/*
 * Kleines Demoprogramm fr das MEGA32 System v2
 * Steuerung ber serielle Schnittstelle wie folgt:
             r0 - Relais aus
             r1 - Relais ein
             shhmmss - Uhr stellen
             t  - Zeit seriell bertragen
             d  - Inhalt des EEProms (24cxx) seriell downloaden
             p0 - PCF Alles aus
             P1 - PCF Alles ein
             w  - Text, welcher ber serielle Schnittstelle empfangen wird, in EEPROM
                  speichern (10 ticks timeout !!) 
*/
int main(void) {
   unsigned int c;
    init();
   while (1) {
     while (ticks < 10) {
       checkTaster();
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
          switch (c) {
            case 'r': toggle_relais(); break;
            case 's': setClock(); break;            
            case 't': printTime(1); break;
            case 'd': downloadEEprom(); break;
            case 'p': toggle_pcf(); break;            
            case 'w': serialProgram(); break;
          }
       }
     }
     ticks = 0;
     lcd_gotoxy(8, 0);
     printTime(0);
     uart_puts_P("DELTAWAVE AVR Developer !\r\n");
   }
}
